@@ -100,12 +100,13 @@ import Text.Pandoc.Compat.Except
 readDocxWithWarnings :: ReaderOptions
                      -> B.ByteString
                      -> Either PandocError (Pandoc, MediaBag, [String])
-readDocxWithWarnings opts bytes =
-  case archiveToDocxWithWarnings (toArchive bytes) of
-    Right (docx, warnings) -> do
-      (meta, blks, mediaBag) <- docxToOutput opts docx
-      return (Pandoc meta blks, mediaBag, warnings)
-    Left _   -> Left (ParseFailure "couldn't parse docx file")
+readDocxWithWarnings opts bytes
+  | Right archive <- toArchiveOrFail bytes
+  , Right (docx, parserWarnings) <- archiveToDocxWithWarnings archive = do
+      (meta, blks, mediaBag, warnings) <- docxToOutput opts docx
+      return (Pandoc meta blks, mediaBag, parserWarnings ++ warnings)
+readDocxWithWarnings _ _ =
+  Left (ParseFailure "couldn't parse docx file")
 
 readDocx :: ReaderOptions
          -> B.ByteString
@@ -117,12 +118,14 @@ readDocx opts bytes = do
 data DState = DState { docxAnchorMap :: M.Map String String
                      , docxMediaBag      :: MediaBag
                      , docxDropCap       :: Inlines
+                     , docxWarnings      :: [String]
                      }
 
 instance Default DState where
   def = DState { docxAnchorMap = M.empty
                , docxMediaBag  = mempty
                , docxDropCap   = mempty
+               , docxWarnings  = []
                }
 
 data DEnv = DEnv { docxOptions  :: ReaderOptions
@@ -136,6 +139,11 @@ type DocxContext = ExceptT PandocError (ReaderT DEnv (State DState))
 evalDocxContext :: DocxContext a -> DEnv -> DState -> Either PandocError a
 evalDocxContext ctx env st = flip evalState st . flip runReaderT env . runExceptT $ ctx
 
+addDocxWarning :: String -> DocxContext ()
+addDocxWarning msg = do
+  warnings <- gets docxWarnings
+  modify $ \s -> s {docxWarnings = msg : warnings}
+
 -- This is empty, but we put it in for future-proofing.
 spansToKeep :: [String]
 spansToKeep = []
@@ -318,6 +326,17 @@ extentToAttr (Just (w, h)) =
     showDim d = show (d / 914400) ++ "in"
 extentToAttr _ = nullAttr
 
+blocksToInlinesWarn :: String -> Blocks -> DocxContext Inlines
+blocksToInlinesWarn cmtId blks = do
+  let blkList = toList blks
+      notParaOrPlain :: Block -> Bool
+      notParaOrPlain (Para _) = False
+      notParaOrPlain (Plain _) = False
+      notParaOrPlain _ = True
+  when (not $ null $ filter notParaOrPlain blkList)
+    (addDocxWarning $ "Docx comment " ++ cmtId ++ " will not retain formatting")
+  return $ fromList $ blocksToInlines blkList
+
 parPartToInlines :: ParPart -> DocxContext Inlines
 parPartToInlines (PlainRun r) = runToInlines r
 parPartToInlines (Insertion _ author date runs) = do
@@ -338,6 +357,22 @@ parPartToInlines (Deletion _ author date runs) = do
       ils <- smushInlines <$> mapM runToInlines runs
       let attr = ("", ["deletion"], [("author", author), ("date", date)])
       return $ spanWith attr ils
+parPartToInlines (CommentStart cmtId author date bodyParts) = do
+  opts <- asks docxOptions
+  case readerTrackChanges opts of
+    AllChanges -> do
+      blks <- smushBlocks <$> mapM bodyPartToBlocks bodyParts
+      ils <- blocksToInlinesWarn cmtId blks
+      let attr = ("", ["comment-start"], [("id", cmtId), ("author", author), ("date", date)])
+      return $ spanWith attr ils
+    _ -> return mempty
+parPartToInlines (CommentEnd cmtId) = do
+  opts <- asks docxOptions
+  case readerTrackChanges opts of
+    AllChanges -> do
+      let attr = ("", ["comment-end"], [("id", cmtId)])
+      return $ spanWith attr mempty
+    _ -> return mempty
 parPartToInlines (BookMark _ anchor) | anchor `elem` dummyAnchors =
   return mempty
 parPartToInlines (BookMark _ anchor) =
@@ -564,18 +599,20 @@ rewriteLink' il = return il
 rewriteLinks :: [Block] -> DocxContext [Block]
 rewriteLinks = mapM (walkM rewriteLink')
 
-bodyToOutput :: Body -> DocxContext (Meta, [Block], MediaBag)
+bodyToOutput :: Body -> DocxContext (Meta, [Block], MediaBag, [String])
 bodyToOutput (Body bps) = do
   let (metabps, blkbps) = sepBodyParts bps
   meta <- bodyPartsToMeta metabps
   blks <- smushBlocks <$> mapM bodyPartToBlocks blkbps
   blks' <- rewriteLinks $ blocksToDefinitions $ blocksToBullets $ toList blks
   mediaBag <- gets docxMediaBag
+  warnings <- gets docxWarnings
   return $ (meta,
             blks',
-            mediaBag)
+            mediaBag,
+            warnings)
 
-docxToOutput :: ReaderOptions -> Docx -> Either PandocError (Meta, [Block], MediaBag)
+docxToOutput :: ReaderOptions -> Docx -> Either PandocError (Meta, [Block], MediaBag, [String])
 docxToOutput opts (Docx (Document _ body)) =
   let dEnv   = def { docxOptions  = opts} in
    evalDocxContext (bodyToOutput body) dEnv def
