@@ -556,11 +556,11 @@ inlineCommands = M.fromList $
        tok >>= \lab ->
          pure (link url "" lab))
   , ("includegraphics", do options <- option [] keyvals
-                           src <- unescapeURL <$> braced
+                           src <- unescapeURL . removeDoubleQuotes <$> braced
                            mkImage options src)
   , ("enquote", enquote)
-  , ("cite", citation "cite" AuthorInText False)
-  , ("Cite", citation "cite" AuthorInText False)
+  , ("cite", citation "cite" NormalCitation False)
+  , ("Cite", citation "Cite" NormalCitation False)
   , ("citep", citation "citep" NormalCitation False)
   , ("citep*", citation "citep*" NormalCitation False)
   , ("citeal", citation "citeal" NormalCitation False)
@@ -1396,3 +1396,10 @@ endInclude = do
   co <- braced
   setPosition $ newPos fn (fromMaybe 1 $ safeRead ln) (fromMaybe 1 $ safeRead co)
   return mempty
+
+removeDoubleQuotes :: String -> String
+removeDoubleQuotes ('"':xs) =
+  case reverse xs of
+       '"':ys -> reverse ys
+       _      -> '"':xs
+removeDoubleQuotes xs = xs
