@@ -63,7 +63,7 @@ import Text.Pandoc.Readers.LaTeX ( rawLaTeXInline, rawLaTeXBlock )
 import Text.HTML.TagSoup (parseTags, innerText, fromAttrib, Tag(..))
 import Text.HTML.TagSoup.Match
 import Data.List ( intercalate )
-import Data.Char ( digitToInt, isUpper)
+import Data.Char ( digitToInt, isUpper )
 import Control.Monad ( guard, liftM, when )
 import Text.Pandoc.Compat.Monoid ((<>))
 import Text.Printf
@@ -277,6 +277,7 @@ definitionListStart :: Parser [Char] ParserState Inlines
 definitionListStart = try $ do
   char '-'
   whitespace
+  notFollowedBy newline
   trimInlines . mconcat <$>
     many1Till inline (try (string ":=")) <* optional whitespace
 
@@ -540,8 +541,8 @@ image = try $ do
   let attr = case lookup "style" kvs of
                Just stls -> (ident, cls, pickStylesToKVs ["width", "height"] stls)
                Nothing   -> (ident, cls, kvs)
-  src <- manyTill anyChar' (lookAhead $ oneOf "!(")
-  alt <- option "" (try $ (char '(' >> manyTill anyChar' (char ')')))
+  src <- many1 (noneOf " \t\n\r!(")
+  alt <- option "" $ try $ char '(' *> manyTill anyChar (char ')')
   char '!'
   return $ B.imageWith attr src alt (B.str alt)
 
@@ -594,10 +595,12 @@ specialAttribute = do
     ("justify" <$ try (string "<>")) <|>
     ("right" <$ char '>') <|>
     ("left" <$ char '<')
+  notFollowedBy spaceChar
   return $ addStyle ("text-align:" ++ alignStr)
 
 attribute :: Parser [Char] ParserState (Attr -> Attr)
-attribute = classIdAttr <|> styleAttr <|> langAttr
+attribute = try $
+  (classIdAttr <|> styleAttr <|> langAttr) <* notFollowedBy spaceChar
 
 classIdAttr :: Parser [Char] ParserState (Attr -> Attr)
 classIdAttr = try $ do -- (class class #id)
@@ -637,10 +640,7 @@ simpleInline :: Parser [Char] ParserState t           -- ^ surrounding parser
              -> (Inlines -> Inlines)       -- ^ Inline constructor
              -> Parser [Char] ParserState Inlines   -- ^ content parser (to be used repeatedly)
 simpleInline border construct = try $ do
-  st <- getState
-  pos <- getPosition
-  let afterString = stateLastStrPos st == Just pos
-  guard $ not afterString
+  notAfterString
   border *> notFollowedBy (oneOf " \t\n\r")
   attr <- attributes
   body <- trimInlines . mconcat <$>
