@@ -39,8 +39,10 @@ import Text.Pandoc.Templates
 import Text.Printf ( printf )
 import Network.URI ( isURI, unEscapeString )
 import Data.Aeson (object, (.=), FromJSON)
-import Data.List ( (\\), isInfixOf, stripPrefix, intercalate, intersperse, nub, nubBy )
-import Data.Char ( toLower, isPunctuation, isAscii, isLetter, isDigit, ord )
+import Data.List ( (\\), isInfixOf, stripPrefix, intercalate, intersperse,
+                   nub, nubBy, foldl' )
+import Data.Char ( toLower, isPunctuation, isAscii, isLetter, isDigit,
+                   ord, isAlphaNum )
 import Data.Maybe ( fromMaybe, isJust, catMaybes )
 import qualified Data.Text as T
 import Control.Applicative ((<|>))
@@ -223,7 +225,7 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                       ++ poly ++ "}{##2}}}\n"
                else "\\newcommand{\\text" ++ poly ++ "}[2][]{\\foreignlanguage{"
                       ++ babel ++ "}{#2}}\n" ++
-                    "\\newenvironment{" ++ poly ++ "}[1]{\\begin{otherlanguage}{"
+                    "\\newenvironment{" ++ poly ++ "}[2][]{\\begin{otherlanguage}{"
                       ++ babel ++ "}}{\\end{otherlanguage}}\n"
             )
             -- eliminate duplicates that have same polyglossia name
@@ -308,7 +310,7 @@ toLabel z = go `fmap` stringToLaTeX URLString z
  where go [] = ""
        go (x:xs)
          | (isLetter x || isDigit x) && isAscii x = x:go xs
-         | elem x ("-+=:;." :: String) = x:go xs
+         | elem x ("_-+=:;." :: String) = x:go xs
          | otherwise = "ux" ++ printf "%x" (ord x) ++ go xs
 
 -- | Puts contents into LaTeX command.
@@ -343,7 +345,7 @@ elementToBeamer slideLevel  (Sec lvl _num (ident,classes,kvs) tit elts)
                     not (null $ query hasCodeBlock elts ++ query hasCode elts)
       let frameoptions = ["allowdisplaybreaks", "allowframebreaks",
                           "b", "c", "t", "environment",
-                          "label", "plain", "shrink"]
+                          "label", "plain", "shrink", "standout"]
       let optionslist = ["fragile" | fragile] ++
                         [k | k <- classes, k `elem` frameoptions] ++
                         [k ++ "=" ++ v | (k,v) <- kvs, k `elem` frameoptions]
@@ -409,16 +411,18 @@ blockToLaTeX (Para [Image attr@(ident, _, _) txt (src,'f':'i':'g':':':tit)]) = d
   capt <- inlineListToLaTeX txt
   notes <- gets stNotes
   modify $ \st -> st{ stInMinipage = False, stNotes = [] }
+
   -- We can't have footnotes in the list of figures, so remove them:
   captForLof <- if null notes
                    then return empty
                    else brackets <$> inlineListToLaTeX (walk deNote txt)
   img <- inlineToLaTeX (Image attr txt (src,tit))
   let footnotes = notesToLaTeX notes
-  figure <- refLabel ident $ cr <>
+  lab <- labelFor ident
+  let caption = "\\caption" <> captForLof <> braces capt <> lab
+  figure <- hypertarget ident (cr <>
             "\\begin{figure}[htbp]" $$ "\\centering" $$ img $$
-            ("\\caption" <> captForLof <> braces capt) $$
-            "\\end{figure}" <> cr
+            caption $$ "\\end{figure}" <> cr)
   return $ if inNote
               -- can't have figures in notes
               then "\\begin{center}" $$ img $+$ capt $$ "\\end{center}"
@@ -469,23 +473,27 @@ blockToLaTeX (CodeBlock (identifier,classes,keyvalAttr) str) = do
         st <- get
         let params = if writerListings (stOptions st)
                      then (case getListingsLanguage classes of
-                                Just l  -> [ "language=" ++ l ]
+                                Just l  -> [ "language=" ++ mbBraced l ]
                                 Nothing -> []) ++
                           [ "numbers=left" | "numberLines" `elem` classes
                              || "number" `elem` classes
                              || "number-lines" `elem` classes ] ++
                           [ (if key == "startFrom"
                                 then "firstnumber"
-                                else key) ++ "=" ++ attr |
+                                else key) ++ "=" ++ mbBraced attr |
                                 (key,attr) <- keyvalAttr ] ++
                           (if identifier == ""
                                 then []
                                 else [ "label=" ++ ref ])
 
                      else []
+            mbBraced x = if not (all isAlphaNum x)
+                            then "{" <> x <> "}"
+                            else x
             printParams
                 | null params = empty
-                | otherwise   = brackets $ hcat (intersperse ", " (map text params))
+                | otherwise   = brackets $ hcat (intersperse ", "
+                      (map text params))
         return $ flush ("\\begin{lstlisting}" <> printParams $$ text str $$
                  "\\end{lstlisting}") $$ cr
   let highlightedCodeBlock =
@@ -506,7 +514,8 @@ blockToLaTeX (RawBlock f x)
 blockToLaTeX (BulletList []) = return empty  -- otherwise latex error
 blockToLaTeX (BulletList lst) = do
   incremental <- gets stIncremental
-  let inc = if incremental then "[<+->]" else ""
+  beamer <- writerBeamer `fmap` gets stOptions
+  let inc = if beamer && incremental then "[<+->]" else ""
   items <- mapM listItemToLaTeX lst
   let spacing = if isTightList lst
                    then text "\\tightlist"
@@ -666,8 +675,8 @@ tableCellToLaTeX header (width, align, blocks) = do
                AlignDefault -> "\\raggedright"
   return $ ("\\begin{minipage}" <> valign <>
             braces (text (printf "%.2f\\columnwidth" width)) <>
-            (halign <> "\\strut" <> cr <> cellContents <> cr) <>
-            "\\strut\\end{minipage}") $$
+            (halign <> "\\strut" <> cr <> cellContents <> "\\strut" <> cr) <>
+            "\\end{minipage}") $$
             notesToLaTeX notes
 
 notesToLaTeX :: [Doc] -> Doc
@@ -718,7 +727,7 @@ sectionHeader :: Bool    -- True for unnumbered
               -> State WriterState Doc
 sectionHeader unnumbered ident level lst = do
   txt <- inlineListToLaTeX lst
-  plain <- stringToLaTeX TextString $ foldl (++) "" $ map stringify lst
+  plain <- stringToLaTeX TextString $ concatMap stringify lst
   let noNote (Note _) = Str ""
       noNote x        = x
   let lstNoNotes = walk noNote lst
@@ -755,7 +764,8 @@ sectionHeader unnumbered ident level lst = do
                   -- needed for \paragraph, \subparagraph in quote environment
                   -- see http://tex.stackexchange.com/questions/169830/
                   else empty
-  stuffing' <- refLabel ident $ text ('\\':sectionType) <> stuffing
+  lab <- labelFor ident
+  stuffing' <- hypertarget ident $ text ('\\':sectionType) <> stuffing <> lab
   return $ if level' > 5
               then txt
               else prefix $$ stuffing'
@@ -765,20 +775,22 @@ sectionHeader unnumbered ident level lst = do
                                 braces txtNoNotes
                          else empty
 
--- | Append label to x and wrap in hypertarget
-refLabel :: String -> Doc -> State WriterState Doc
-refLabel ident x = do
+hypertarget :: String -> Doc -> State WriterState Doc
+hypertarget ident x = do
   ref <- text `fmap` toLabel ident
   internalLinks <- gets stInternalLinks
-  let hypertarget y = if ident `elem` internalLinks
-                         then text "\\hypertarget"
-                                <> braces ref
-                                <> braces y
-                         else y
-      label = if null ident
-                 then empty
-                 else text "\\label" <> braces ref
-  return $ hypertarget $ x <> label
+  return $
+    if ident `elem` internalLinks
+       then text "\\hypertarget"
+              <> braces ref
+              <> braces x
+       else x
+
+labelFor :: String -> State WriterState Doc
+labelFor ""    = return empty
+labelFor ident = do
+  ref <- text `fmap` toLabel ident
+  return $ text "\\label" <> braces ref
 
 -- | Convert list of inline elements to LaTeX.
 inlineListToLaTeX :: [Inline]  -- ^ Inlines to convert
@@ -961,7 +973,7 @@ inlineToLaTeX (Image attr _ (source, _)) = do
       source' = if isURI source
                    then source
                    else unEscapeString source
-  source'' <- stringToLaTeX URLString (escapeURI source')
+  source'' <- stringToLaTeX URLString source'
   inHeading <- gets stInHeading
   return $
     (if inHeading then "\\protect\\includegraphics" else "\\includegraphics") <>
@@ -1027,7 +1039,7 @@ citationsToNatbib (c:cs) | citationMode c == AuthorInText = do
 
 citationsToNatbib cits = do
   cits' <- mapM convertOne cits
-  return $ text "\\citetext{" <> foldl combineTwo empty cits' <> text "}"
+  return $ text "\\citetext{" <> foldl' combineTwo empty cits' <> text "}"
   where
     combineTwo a b | isEmpty a = b
                    | otherwise = a <> text "; " <> b
@@ -1076,7 +1088,7 @@ citationsToBiblatex (one:[])
 
 citationsToBiblatex (c:cs) = do
   args <- mapM convertOne (c:cs)
-  return $ text cmd <> foldl (<>) empty args
+  return $ text cmd <> foldl' (<>) empty args
     where
        cmd = case citationMode c of
                   AuthorInText -> "\\textcites"
@@ -1120,7 +1132,7 @@ toPolyglossiaEnv l =
 
 -- Takes a list of the constituents of a BCP 47 language code and
 -- converts it to a Polyglossia (language, options) tuple
--- http://mirrors.concertpass.com/tex-archive/macros/latex/contrib/polyglossia/polyglossia.pdf
+-- http://mirrors.ctan.org/macros/latex/contrib/polyglossia/polyglossia.pdf
 toPolyglossia :: [String] -> (String, String)
 toPolyglossia ("ar":"DZ":_)        = ("arabic", "locale=algeria")
 toPolyglossia ("ar":"IQ":_)        = ("arabic", "locale=mashriq")
@@ -1148,17 +1160,21 @@ toPolyglossia ("en":"UK":_)        = ("english", "variant=british")
 toPolyglossia ("en":"US":_)        = ("english", "variant=american")
 toPolyglossia ("grc":_)            = ("greek",   "variant=ancient")
 toPolyglossia ("hsb":_)            = ("usorbian", "")
+toPolyglossia ("la":"x":"classic":_) = ("latin", "variant=classic")
 toPolyglossia ("sl":_)             = ("slovenian", "")
 toPolyglossia x                    = (commonFromBcp47 x, "")
 
 -- Takes a list of the constituents of a BCP 47 language code and
 -- converts it to a Babel language string.
--- http://mirrors.concertpass.com/tex-archive/macros/latex/required/babel/base/babel.pdf
--- Note that the PDF unfortunately does not contain a complete list of supported languages.
+-- http://mirrors.ctan.org/macros/latex/required/babel/base/babel.pdf
+-- List of supported languages (slightly outdated):
+-- http://tug.ctan.org/language/hyph-utf8/doc/generic/hyph-utf8/hyphenation.pdf
 toBabel :: [String] -> String
 toBabel ("de":"1901":_)      = "german"
 toBabel ("de":"AT":"1901":_) = "austrian"
 toBabel ("de":"AT":_)        = "naustrian"
+toBabel ("de":"CH":"1901":_) = "swissgerman"
+toBabel ("de":"CH":_)        = "nswissgerman"
 toBabel ("de":_)             = "ngerman"
 toBabel ("dsb":_)            = "lowersorbian"
 toBabel ("el":"polyton":_)   = "polutonikogreek"
@@ -1172,6 +1188,7 @@ toBabel ("fr":"CA":_)        = "canadien"
 toBabel ("fra":"aca":_)      = "acadian"
 toBabel ("grc":_)            = "polutonikogreek"
 toBabel ("hsb":_)            = "uppersorbian"
+toBabel ("la":"x":"classic":_) = "classiclatin"
 toBabel ("sl":_)             = "slovene"
 toBabel x                    = commonFromBcp47 x
 
@@ -1180,12 +1197,17 @@ toBabel x                    = commonFromBcp47 x
 -- https://tools.ietf.org/html/bcp47#section-2.1
 commonFromBcp47 :: [String] -> String
 commonFromBcp47 [] = ""
-commonFromBcp47 ("pt":"BR":_) = "brazilian"
+commonFromBcp47 ("pt":"BR":_)            = "brazil"
+-- Note: documentation says "brazilian" works too, but it doesn't seem to work
+-- on some systems.  See #2953.
+commonFromBcp47 ("sr":"Cyrl":_)          = "serbianc"
+commonFromBcp47 ("zh":"Latn":"pinyin":_) = "pinyin"
 commonFromBcp47 x = fromIso $ head x
   where
     fromIso "af"  = "afrikaans"
     fromIso "am"  = "amharic"
     fromIso "ar"  = "arabic"
+    fromIso "as"  = "assamese"
     fromIso "ast" = "asturian"
     fromIso "bg"  = "bulgarian"
     fromIso "bn"  = "bengali"
@@ -1209,12 +1231,13 @@ commonFromBcp47 x = fromIso $ head x
     fromIso "fur" = "friulan"
     fromIso "ga"  = "irish"
     fromIso "gd"  = "scottish"
+    fromIso "gez" = "ethiopic"
     fromIso "gl"  = "galician"
     fromIso "he"  = "hebrew"
     fromIso "hi"  = "hindi"
     fromIso "hr"  = "croatian"
-    fromIso "hy"  = "armenian"
     fromIso "hu"  = "magyar"
+    fromIso "hy"  = "armenian"
     fromIso "ia"  = "interlingua"
     fromIso "id"  = "indonesian"
     fromIso "ie"  = "interlingua"
@@ -1222,6 +1245,7 @@ commonFromBcp47 x = fromIso $ head x
     fromIso "it"  = "italian"
     fromIso "jp"  = "japanese"
     fromIso "km"  = "khmer"
+    fromIso "kmr" = "kurmanji"
     fromIso "kn"  = "kannada"
     fromIso "ko"  = "korean"
     fromIso "la"  = "latin"
@@ -1237,6 +1261,7 @@ commonFromBcp47 x = fromIso $ head x
     fromIso "no"  = "norsk"
     fromIso "nqo" = "nko"
     fromIso "oc"  = "occitan"
+    fromIso "pa"  = "panjabi"
     fromIso "pl"  = "polish"
     fromIso "pms" = "piedmontese"
     fromIso "pt"  = "portuguese"
@@ -1253,6 +1278,7 @@ commonFromBcp47 x = fromIso $ head x
     fromIso "ta"  = "tamil"
     fromIso "te"  = "telugu"
     fromIso "th"  = "thai"
+    fromIso "ti"  = "ethiopic"
     fromIso "tk"  = "turkmen"
     fromIso "tr"  = "turkish"
     fromIso "uk"  = "ukrainian"
@@ -1283,4 +1309,3 @@ pDocumentClass =
        else do P.skipMany (P.satisfy (/='{'))
                P.char '{'
                P.manyTill P.letter (P.char '}')
-                                   
