@@ -1,6 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-
 Copyright (C) 2010-2015 Puneeth Chaganti <punchagan@gmail.com>
+                        Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>,
                         and John MacFarlane <jgm@berkeley.edu>
 
 This program is free software; you can redistribute it and/or modify
@@ -38,7 +39,8 @@ import Text.Pandoc.Shared
 import Text.Pandoc.Writers.Shared
 import Text.Pandoc.Pretty
 import Text.Pandoc.Templates (renderTemplate')
-import Data.List ( intersect, intersperse, transpose )
+import Data.Char ( toLower )
+import Data.List ( intersect, intersperse, partition, transpose )
 import Control.Monad.State
 
 data WriterState =
@@ -110,14 +112,47 @@ isRawFormat f =
 blockToOrg :: Block         -- ^ Block element
            -> State WriterState Doc
 blockToOrg Null = return empty
+blockToOrg (Div (_,classes@(cls:_),kvs) bs) | "drawer" `elem` classes = do
+  contents <- blockListToOrg bs
+  let drawerNameTag = ":" <> text cls <> ":"
+  let keys = vcat $ map (\(k,v) ->
+                       ":" <> text k <> ":"
+                       <> space <> text v) kvs
+  let drawerEndTag = text ":END:"
+  return $ drawerNameTag $$ cr $$ keys $$
+           blankline $$ contents $$
+           blankline $$ drawerEndTag $$
+           blankline
 blockToOrg (Div attrs bs) = do
   contents <- blockListToOrg bs
-  let startTag = tagWithAttrs "div" attrs
-  let endTag = text "</div>"
-  return $ blankline $$ "#+BEGIN_HTML" $$
-           nest 2 startTag $$ "#+END_HTML" $$ blankline $$
-           contents $$ blankline $$ "#+BEGIN_HTML" $$
-           nest 2 endTag $$ "#+END_HTML" $$ blankline
+  let isGreaterBlockClass = (`elem` ["center", "quote"]) . map toLower
+  return $ case attrs of
+    ("", [], []) ->
+      -- nullAttr, treat contents as if it wasn't wrapped
+      blankline $$ contents $$ blankline
+    (ident, [], []) ->
+      -- only an id: add id as an anchor, unwrap the rest
+      blankline $$ "<<" <> text ident <> ">>" $$ contents $$ blankline
+    (ident, classes, kv) ->
+      -- if one class looks like the name of a greater block then output as
+      -- such: The ID, if present, is added via the #+NAME keyword; other
+      -- classes and key-value pairs are kept as #+ATTR_HTML attributes.
+      let
+        (blockTypeCand, classes') = partition isGreaterBlockClass classes
+      in case blockTypeCand of
+        (blockType:classes'') ->
+          blankline $$ attrHtml (ident, classes'' <> classes', kv) $$
+          "#+BEGIN_" <> text blockType $$ contents $$
+          "#+END_" <> text blockType $$ blankline
+        _                     ->
+          -- fallback: wrap in div tags
+          let
+            startTag = tagWithAttrs "div" attrs
+            endTag = text "</div>"
+          in blankline $$ "#+BEGIN_HTML" $$
+             nest 2 startTag $$ "#+END_HTML" $$ blankline $$
+             contents $$ blankline $$ "#+BEGIN_HTML" $$
+             nest 2 endTag $$ "#+END_HTML" $$ blankline
 blockToOrg (Plain inlines) = inlineListToOrg inlines
 -- title beginning with fig: indicates that the image is a figure
 blockToOrg (Para [Image attr txt (src,'f':'i':'g':':':tit)]) = do
@@ -137,10 +172,13 @@ blockToOrg (RawBlock f str) | isRawFormat f =
   return $ text str
 blockToOrg (RawBlock _ _) = return empty
 blockToOrg HorizontalRule = return $ blankline $$ "--------------" $$ blankline
-blockToOrg (Header level _ inlines) = do
+blockToOrg (Header level attr inlines) = do
   contents <- inlineListToOrg inlines
   let headerStr = text $ if level > 999 then " " else replicate level '*'
-  return $ headerStr <> " " <> contents <> blankline
+  let drawerStr = if attr == nullAttr
+                  then empty
+                  else cr <> nest (level + 1) (propertiesDrawer attr)
+  return $ headerStr <> " " <> contents <> drawerStr <> blankline
 blockToOrg (CodeBlock (_,classes,_) str) = do
   opts <- stOptions <$> get
   let tabstop = writerTabStop opts
@@ -170,7 +208,7 @@ blockToOrg (Table caption' _ _ headers rows) =  do
        map ((+2) . numChars) $ transpose (headers' : rawRows)
   -- FIXME: Org doesn't allow blocks with height more than 1.
   let hpipeBlocks blocks = hcat [beg, middle, end]
-        where h      = maximum (map height blocks)
+        where h      = maximum (1 : map height blocks)
               sep'   = lblock 3 $ vcat (map text $ replicate h " | ")
               beg    = lblock 2 $ vcat (map text $ replicate h "| ")
               end    = lblock 2 $ vcat (map text $ replicate h " |")
@@ -230,6 +268,32 @@ definitionListItemToOrg (label, defs) = do
   contents <- liftM vcat $ mapM blockListToOrg defs
   return $ hang 3 "-  " $ label' <> " :: " <> (contents <> cr)
 
+-- | Convert list of key/value pairs to Org :PROPERTIES: drawer.
+propertiesDrawer :: Attr -> Doc
+propertiesDrawer (ident, classes, kv) =
+  let
+    drawerStart = text ":PROPERTIES:"
+    drawerEnd   = text ":END:"
+    kv'  = if (classes == mempty) then kv  else ("CLASS", unwords classes):kv
+    kv'' = if (ident == mempty)   then kv' else ("CUSTOM_ID", ident):kv'
+    properties = vcat $ map kvToOrgProperty kv''
+  in
+    drawerStart <> cr <> properties <> cr <> drawerEnd
+ where
+   kvToOrgProperty :: (String, String) -> Doc
+   kvToOrgProperty (key, value) =
+     text ":" <> text key <> text ": " <> text value <> cr
+
+attrHtml :: Attr -> Doc
+attrHtml (""   , []     , []) = mempty
+attrHtml (ident, classes, kvs) =
+  let
+    name = if (null ident) then mempty else "#+NAME: " <> text ident <> cr
+    keyword = "#+ATTR_HTML"
+    classKv = ("class", unwords classes)
+    kvStrings = map (\(k,v) -> ":" <> k <> " " <> v) (classKv:kvs)
+  in name <> keyword <> ": " <> text (unwords kvStrings) <> cr
+
 -- | Convert list of Pandoc block elements to Org.
 blockListToOrg :: [Block]       -- ^ List of block elements
                -> State WriterState Doc
@@ -275,9 +339,10 @@ inlineToOrg (Math t str) = do
   return $ if t == InlineMath
               then "$" <> text str <> "$"
               else "$$" <> text str <> "$$"
-inlineToOrg (RawInline f str) | isRawFormat f =
-  return $ text str
-inlineToOrg (RawInline _ _) = return empty
+inlineToOrg (RawInline f@(Format f') str) =
+  return $ if isRawFormat f
+           then text str
+           else "@@" <> text f' <> ":" <> text str <> "@@"
 inlineToOrg (LineBreak) = return (text "\\\\" <> cr)
 inlineToOrg Space = return space
 inlineToOrg SoftBreak = do
