@@ -1,5 +1,5 @@
 {-
-Copyright (C) 2006-2014 John MacFarlane <jgm@berkeley.edu>
+Copyright (C) 2006-2015 John MacFarlane <jgm@berkeley.edu>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 {- |
    Module      : Text.Pandoc.Writers.RTF
-   Copyright   : Copyright (C) 2006-2014 John MacFarlane
+   Copyright   : Copyright (C) 2006-2015 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -46,7 +46,7 @@ import Text.Pandoc.ImageSize
 -- or a MediaBag, or the internet.
 -- If file not found or filetype not jpeg or png, leave the inline unchanged.
 rtfEmbedImage :: WriterOptions -> Inline -> IO Inline
-rtfEmbedImage opts x@(Image _ (src,_)) = do
+rtfEmbedImage opts x@(Image attr _ (src,_)) = do
   result <- fetchItem' (writerMediaBag opts) (writerSourceURL opts) src
   case result of
        Right (imgdata, Just mime)
@@ -56,16 +56,19 @@ rtfEmbedImage opts x@(Image _ (src,_)) = do
                              "image/jpeg" -> "\\jpegblip"
                              "image/png"  -> "\\pngblip"
                              _            -> error "Unknown file type"
-         let sizeSpec = case imageSize imgdata of
-                             Nothing -> ""
-                             Just sz -> "\\picw" ++ show xpx ++
+         sizeSpec <- case imageSize imgdata of
+                             Left msg -> do
+                               warn $ "Could not determine image size in `" ++
+                                 src ++ "': " ++ msg
+                               return ""
+                             Right sz -> return $ "\\picw" ++ show xpx ++
                                         "\\pich" ++ show ypx ++
-                                        "\\picwgoal" ++ show (xpt * 20)
-                                        ++ "\\pichgoal" ++ show (ypt * 20)
+                                        "\\picwgoal" ++ show (floor (xpt * 20) :: Integer)
+                                        ++ "\\pichgoal" ++ show (floor (ypt * 20) :: Integer)
                                 -- twip = 1/1440in = 1/20pt
                                 where (xpx, ypx) = sizeInPixels sz
-                                      (xpt, ypt) = sizeInPoints sz
-         let raw = "{\\pict" ++ filetype ++ sizeSpec ++ " " ++
+                                      (xpt, ypt) = desiredSizeInPoints opts attr sz
+         let raw = "{\\pict" ++ filetype ++ sizeSpec ++ "\\bin " ++
                     concat bytes ++ "}"
          return $ if B.null imgdata
                      then x
@@ -346,11 +349,12 @@ inlineToRTF (RawInline f str)
   | f == Format "rtf" = str
   | otherwise         = ""
 inlineToRTF (LineBreak) = "\\line "
+inlineToRTF SoftBreak = " "
 inlineToRTF Space = " "
-inlineToRTF (Link text (src, _)) =
+inlineToRTF (Link _ text (src, _)) =
   "{\\field{\\*\\fldinst{HYPERLINK \"" ++ (codeStringToRTF src) ++
   "\"}}{\\fldrslt{\\ul\n" ++ (inlineListToRTF text) ++ "\n}}}\n"
-inlineToRTF (Image _ (source, _)) =
+inlineToRTF (Image _ _ (source, _)) =
   "{\\cf1 [image: " ++ source ++ "]\\cf0}"
 inlineToRTF (Note contents) =
   "{\\super\\chftn}{\\*\\footnote\\chftn\\~\\plain\\pard " ++
