@@ -49,21 +49,24 @@ arbInline n = frequency $ [ (60, liftM Str realString)
                    , (10,  liftM Strikeout $ arbInlines (n-1))
                    , (10,  liftM Superscript $ arbInlines (n-1))
                    , (10,  liftM Subscript $ arbInlines (n-1))
---                   , (10,  liftM SmallCaps $ arbInlines (n-1))
+                   , (10,  liftM SmallCaps $ arbInlines (n-1))
                    , (10,  do x1 <- arbitrary
                               x2 <- arbInlines (n-1)
                               return $ Quoted x1 x2)
                    , (10,  do x1 <- arbitrary
                               x2 <- realString
                               return $ Math x1 x2)
-                   , (10,  do x1 <- arbInlines (n-1)
+                   , (10,  do x0 <- arbAttr
+                              x1 <- arbInlines (n-1)
                               x3 <- realString
                               x2 <- liftM escapeURI realString
-                              return $ Link x1 (x2,x3))
-                   , (10,  do x1 <- arbInlines (n-1)
+                              return $ Link x0 x1 (x2,x3))
+                   , (10,  do x0 <- arbAttr
+                              x1 <- arbInlines (n-1)
                               x3 <- realString
                               x2 <- liftM escapeURI realString
-                              return $ Image x1 (x2,x3))
+                              return $ Image x0 x1 (x2,x3))
+                   , (2,  liftM2 Cite arbitrary (arbInlines 1))
                    , (2,  liftM Note $ resize 3 $ listOf1 $ arbBlock (n-1))
                    ]
 
@@ -111,7 +114,6 @@ instance Arbitrary Pandoc where
         arbitrary = resize 8 $ liftM normalize
                              $ liftM2 Pandoc arbitrary arbitrary
 
-{-
 instance Arbitrary CitationMode where
         arbitrary
           = do x <- choose (0 :: Int, 2)
@@ -123,14 +125,13 @@ instance Arbitrary CitationMode where
 
 instance Arbitrary Citation where
         arbitrary
-          = do x1 <- liftM (filter (`notElem` ",;]@ \t\n")) arbitrary
-               x2 <- arbitrary
-               x3 <- arbitrary
+          = do x1 <- listOf $ elements $ ['a'..'z'] ++ ['0'..'9'] ++ ['_']
+               x2 <- arbInlines 1
+               x3 <- arbInlines 1
                x4 <- arbitrary
                x5 <- arbitrary
                x6 <- arbitrary
                return (Citation x1 x2 x3 x4 x5 x6)
--}
 
 instance Arbitrary MathType where
         arbitrary
