@@ -57,7 +57,7 @@ tests = [ testGroup "markdown"
               "tables.txt" "tables.native"
             , test "pipe tables" ["-r", "markdown", "-w", "native", "--columns=80"]
               "pipe-tables.txt" "pipe-tables.native"
-            , test "more" ["-r", "markdown", "-w", "native", "-S"]
+            , test "more" ["-r", "markdown", "-w", "native", "-s", "-S"]
               "markdown-reader-more.txt" "markdown-reader-more.native"
             , lhsReaderTest "markdown+lhs"
             ]
@@ -108,6 +108,9 @@ tests = [ testGroup "markdown"
           , test "reader" ["-r", "docbook", "-w", "native", "-s"]
             "docbook-xref.docbook" "docbook-xref.native"
           ]
+        , testGroup "docbook5"
+          [ testGroup "writer" $ writerTests "docbook5"
+          ]
         , testGroup "native"
           [ testGroup "writer" $ writerTests "native"
           , test "reader" ["-r", "native", "-w", "native", "-s"]
@@ -163,7 +166,7 @@ tests = [ testGroup "markdown"
               "twiki-reader.twiki" "twiki-reader.native" ]
         , testGroup "other writers" $ map (\f -> testGroup f $ writerTests f)
           [ "opendocument" , "context" , "texinfo", "icml", "tei"
-          , "man" , "plain" , "rtf", "org", "asciidoc"
+          , "man" , "plain" , "rtf", "org", "asciidoc", "zimwiki"
           ]
         , testGroup "writers-lang-and-dir"
           [ test "latex" ["-f", "native", "-t", "latex", "-s"]
