@@ -7,7 +7,7 @@ import Text.Pandoc
 import Tests.Helpers
 import Tests.Arbitrary()
 
-html :: (ToString a, ToPandoc a) => a -> String
+html :: (ToPandoc a) => a -> String
 html = writeHtmlString def{ writerWrapText = WrapNone } . toPandoc
 
 {-
